%%************************************************************************
%% Program: VSTMCapacity.m      Programmer: Mike Ambinder
%% Vision Lab Research   University of Illinois at Urbana-Champaign
%% Created: 4/30/2008           Last Modified: 5/4/2008
%%************************************************************************
%% This program has participants run in the basic Luck & Vogel (1997)
%% color change detection task.  A setsize of 6 is used in all trials, and
%% K is computed at the end of the experiment.
%%************************************************************************
%% Please run in the command window with the -nojvm option
%% Please run at 60 Hz
%%************************************************************************
%% Depends on: previously, Inst.m and instructions file
%%                 

% Modified 02.13.12 - K. Divis: For PTB3, needs WriteLine.m &
% WriteCentered.m 

%%************************************************************************
%% Initialization
%%************************************************************************
clear all;
close all;

WaitSecs(0.5);                                              %% get MEX file loaded

subNum = input('Please enter the subject number: ');            %% get subject number

while true
    if ~exist(['VSTMcapacity_' num2str(subNum) '.mat'], 'file')
        break
    else
        subNum = input('That subject already exists.  Please enter a new subject number.\n');
    end
end
% Inst VSTMCapacity                                           %% display instructions
% load tempblock;
% load tempSbjID;
    
% global window;
global rect;
global XLeft;
global XRight;
global YTop;
global YBottom;
global XCentre; 
global YCentre;
% global blankScreen;
global spaceBar;


rand('state',sum(100*clock));                               %% set random number key

           
[w,rect] = Screen('OpenWindow',2,[127.5,127.5,127.5]);      %% create main window

HideCursor;                                                 %% what it says

XLeft   = rect(RectLeft);	XRight  = rect(RectRight);      %% get screen coordinates
YTop    = rect(RectTop);	YBottom = rect(RectBottom);
XCentre = XRight./2; 		YCentre = YBottom./2;

Screen('TextSize',w,20);                               %% set text size
Screen('TextFont',w,'Gilsans');                        %% set text font

%%************************************************************************
%% Structures and Variables
%%************************************************************************

%% Utility variables
TRUE  = 1;                                                  %% use for conditionals
FALSE = 0;
i = 1;                                                      %% counter variables
j = 1;
REFRESHRATE = 85;                                           %% check to ensure monitor is set at 60 Hz --- CHANGED TO 85!!!
escapeKey   = KbName('esc');                                %% 'escape' key exits program
spaceBar    = KbName('space');                              %% spacebar moves on from menu screens                   
z_key       = KbName('z');                                  %% 'z' responds 'same'        
slash_key   = KbName('/?');                                 %% '/' responds 'different'

%% Colors
white   = [255,255,255];                                    %% RGB values for different colors   
black   = [0,0,0];
gray    = (white+black)./2;
red     = [255,0,0];
green   = [0,255,0];
blue    = [0,0,255];
yellow  = [255,255,0];
purple  = [160,32,240];

%% Experiment Variables
numBlocks = 1;                                              %% number of blocks in the experiment (1)
blockTrials = 100;                                          %% number of trials per block (100)             --CHANGED!!!
blockPracticeTrials = 4;                                    %% number of practice trials per block (4)
practiceTrials = numBlocks * blockPracticeTrials;           %% total number of practice trials (4)
experTrials = numBlocks * blockTrials;                      %% total number of experimental trials (125)
numTrials = practiceTrials + experTrials;                   %% number of trials (129)
currBlockTrial = 0;                                         %% current trial in the block
currBlock = 0;                                              %% current block
currTrial = 0;                                              %% current overall trial
objectProportion = 0.065;                                   %% relative size of stimulus
objectSize = objectProportion * XRight;                     %% default object size (in pixels)
jitter = 0.20*objectSize;                                   %% randomization value for object positions
objectRect = [0 0 objectSize objectSize];                   %% rectangle coordinates of stimuli
objectBuffer = objectSize*1.5;                              %% minimum space between objects
bufferProportion = 0.1;                                     %% proportion of screen to use as a buffer
screenBuffer = bufferProportion * XRight;                   %% area around screen where no objects presented (in pixels)
instructionTime = 15;
ScreenHeight = RectHeight(rect);
ScreenWidth = RectWidth(rect);
xPos = ScreenWidth/2;
yPos = ScreenHeight/2;
%% Trial Variables
setsize = 6;                                                %% set setsize
change = 0;                                                 %% same (0) or different (1) trial
response = 0;                                               %% participant's response (0 = no change; 1 = change)
startTime = GetSecs;                                        %% start of respone window (also load MEX file)
endTime = GetSecs;                                          %% end of response window
responseTime = 0;                                           %% response time
hits = 0;                                                   %% successful change detection (used to computer K)
misses = 0;                                                 %% unsuccessful change detection (used to computer K)
falseAlarms = 0;                                            %% nonexistent change detection (used to computer K)
correctRejections = 0;                                      %% successful detection of no change (used to computer K)
capacity = 0;                                               %% store value of K

colorSet = {white,white,black,black,red,red,blue,blue ...   %% colors to use in the experiment (cell array)
            green,green,yellow,yellow,purple,purple}; ...

trials(numTrials) = struct('SbjID',0,'block',0,      ...   %% structure to store trial information
    'blockTrial',0,'trial',0,'setsize',0,'change',0,  ...
    'response',0,'RT',0,'correct',0,'hit',0,'miss',0, ...
    'falseAlarm',0,'correctRejection',0);

%%************************************************************************
%% Prepare Stimuli
%%************************************************************************

%% Create Stimuli
% blankScreen = Screen(window,'OpenOffscreenWindow',gray,rect);           %% blank screen
% probeScreen = Screen(window,'OpenOffscreenWindow',gray,rect);           %% initial display
% testScreen  = Screen(window,'OpenOffscreenWindow',gray,rect);           %% test display

blank_GrayMatrix=ones(rect(4),rect(3),3).*gray(1);
blankScreen = Screen('MakeTexture',w,blank_GrayMatrix);
probeScreen = Screen('MakeTexture',w,blank_GrayMatrix);
testScreen = Screen('MakeTexture',w,blank_GrayMatrix);

%% Generate Display Positions
xCoord = screenBuffer;                                                  %% start at left edge of screen
yCoord = screenBuffer;                                                  %% start at top edge of screen
i = 1;                                                                  %% index
while xCoord < (XRight - screenBuffer)                                  %% go from left to right
    xPositions(i) = xCoord;                                             %% store x-coordinate of possible positions
    xCoord = xCoord + objectBuffer;                                     %% go to next position
    i = i + 1;                                                          %% increase index
end  %% end while loop
j = 1;                                                                  %% index
while yCoord < (YBottom - screenBuffer)                                 %% go from top to bottom
    yPositions(j) = yCoord;                                             %% store y-coordinate of possible positions
    yCoord = yCoord + objectBuffer;                                     %% go to next position
    j = j + 1;                                                          %% increase index
end %% end while loop

%% Prepare Trials
for i = 1:numTrials                                                     %% store trial data
    
    xPositionsTemp = Shuffle(xPositions);                                           %% randomize x-coordinates
    yPositionsTemp = Shuffle(yPositions);                                           %% randomize y-coordinates
    for j = 1:length(xPositionsTemp)                                                %% add random jitter to x-positions
        if round(rand(1))                                                           %% get a random sign
            sign = -1;
        else
            sign = 1;
        end
        xPositionsTemp(j) = xPositionsTemp(j) + round((sign*(jitter*(rand(1)))));   %% get some random x-coordinate jitter
    end %% end for loop
    for k = 1:length(yPositionsTemp)                                                %% add random jitter to y-positions
        if round(rand(1))                                                           %% get a random sign
            sign = -1;
        else
            sign = 1;
        end
        yPositionsTemp(k) = yPositionsTemp(k) + round((sign*(jitter*(rand(1)))));   %% get some random y-coordinate jitter
    end  %% end for loop
    
    colorSet                = Shuffle(colorSet);                        %% randomize color set
    trials(i).SbjID         = subNum;                                   %% store subject number
    trials(i).change        = mod(i,2);                                 %% half are change trials; half aren't
    trials(i).setsize       = setsize;                                  %% setsize is constant
    trials(i).pos1          = [xPositionsTemp(1),yPositionsTemp(1)];    %% get coordinates of first stimulus
    trials(i).pos2          = [xPositionsTemp(2),yPositionsTemp(2)];    %% get coordinates of second stimulus
    trials(i).pos3          = [xPositionsTemp(3),yPositionsTemp(3)];    %% get coordinates of third stimulus
    trials(i).pos4          = [xPositionsTemp(4),yPositionsTemp(4)];    %% get coordinates of fourth stimulus
    trials(i).pos5          = [xPositionsTemp(5),yPositionsTemp(5)];    %% get coordinates of fifth stimulus
    trials(i).pos6          = [xPositionsTemp(6),yPositionsTemp(6)];    %% get coordinates of sixth stimulus
    trials(i).color1        = colorSet{1};                              %% get color of first stimulus
    trials(i).color2        = colorSet{2};                              %% get color of second stimulus
    trials(i).color3        = colorSet{3};                              %% get color of third stimulus
    trials(i).color4        = colorSet{4};                              %% get color of fourth stimulus
    trials(i).color5        = colorSet{5};                              %% get color of fifth stimulus
    trials(i).color6        = colorSet{6};                              %% get color of sixth stimulus (will be object that changes color if necessary)
    trials(i).changeColor   = colorSet{7};                              %% get color of change stimulus (if necessary)
    if strcmp(trials(i).color6,trials(i).changeColor)                   %% make sure change color is not equivalent to probe display color
        trials(i).changeColor = colorSet(8);                            %% only 2 of same color possible in display, so color 8 will be different
    end  %% end if clause
    
end  %% end for loop
trials = Shuffle(trials);                                               %% randomize trials


%%************************************************************************
%% Run Program
%%************************************************************************

%Initial instructions
Screen('TextSize',w, 25);
Screen('TextFont',w, 'Gilsans');
WriteLine(w, 'In this portion of the experiment, you will be asked to make judgments about whether or not the display changes. On each trial, 6 colored squares will briefly appear, followed by a blank screen, and then 6 more colored squares. Your task is to determine whether one of those squares CHANGED COLOR from the first display to the second. If the second display is the SAME as the first display (i.e., all the squares stayed the same color), press the ''Z'' key. If the second dispaly was DIFFERENT from the first display (i.e., one of the squares changed color), press the ''/'' key. Keep your left index finger on the ''Z'' key and your right index finger on the ''/'' key and at all times. Please try to respond as quickly and accurately as possible.|| Remember, hit ''Z'' for same and ''/'' for different.' , black, 100, 175, 75, 1.5);
Screen('Flip',w,[],1);
pause(instructionTime);
Screen('TextSize',w, 15);
WriteCentered(w, 'If you have any questions, ask the experimenter now.' , xPos, ScreenHeight-140, black);
WriteCentered(w, 'Otherwise, press the ''RETURN'' key when you are ready to continue.' , xPos, ScreenHeight-100, black);
Screen('Flip',w);
wait = KbWait;
Screen('TextSize',w, 20);

currBlock = 0;                                                          %% 0 = practice trials, 1 = experimental trials    
currBlockTrial = 1;                                                     %% start at 1st trial in block
currTrial = 1;                                                          %% start at first overall trial
hits = 0;                                                               %% reset SDT counters
misses = 0;
falseAlarms = 0;
correctRejections = 0;
for i = 1:numTrials                                                     %% run trials
    %Screen('CopyWindow',blankScreen,window);
    Screen('DrawTexture',w,blankScreen);
    Screen('Flip',w);
    
    trials(i).block         = currBlock;                                %% store current block
    trials(i).blockTrial    = currBlockTrial;                           %% store current block trial
    trials(i).trial         = currTrial;                                %% store current trial

    %% Create Probe and Test Displays
    
    %SCREEN('CopyWindow',blankScreen,probeScreen);                       %% clear probe display
    %SCREEN('CopyWindow',blankScreen,testScreen);                       
    
    Screen('DrawTexture',probeScreen,blankScreen);                      %% clear probe display
    Screen('DrawTexture',testScreen,blankScreen);                       %% clear test display
    
    SCREEN('FillRect',probeScreen,trials(i).color1,objectRect+ ...      %% place first item on probe screen
          [trials(i).pos1 trials(i).pos1]);
    SCREEN('FillRect',probeScreen,trials(i).color2,objectRect+ ...      %% place second item on probe screen
          [trials(i).pos2 trials(i).pos2]);
    SCREEN('FillRect',probeScreen,trials(i).color3,objectRect+ ...      %% place third item on probe screen
          [trials(i).pos3 trials(i).pos3]);
    SCREEN('FillRect',probeScreen,trials(i).color4,objectRect+ ...      %% place fourth item on probe screen
          [trials(i).pos4 trials(i).pos4]);
    SCREEN('FillRect',probeScreen,trials(i).color5,objectRect+ ...      %% place fifth item on probe screen
          [trials(i).pos5 trials(i).pos5]);
    SCREEN('FillRect',probeScreen,trials(i).color6,objectRect+ ...      %% place sixth item on probe screen
          [trials(i).pos6 trials(i).pos6]);
    
    
      
    %SCREEN('CopyWindow',probeScreen,testScreen);                        %% create test display
    Screen('DrawTexture',testScreen,probeScreen);                       %% create test display
    
    if trials(i).change                                                 %% add change object if needed
        SCREEN('FillRect',testScreen,trials(i).changeColor,    ...      %% place change object in position 6
               objectRect+[trials(i).pos6 trials(i).pos6]);
    end %% end if clause
    
    %% Run trial
    
    message='Please press the spacebar to start the next trial';
    Screen('DrawText',w,message,XCentre./2,YCentre,black);       %% trial start message
    Screen('Flip',w);
    
    while 1
        [down time key]= KbCheck;
        if key(spaceBar)
            break;
        end
    end
    
%     while KbCheck; end;                                               %% flush keyboard buffer
    trialStart=GetSecs;
%     WaitSecs(0.5);                                                    %% 500ms blank before start of trial
    while GetSecs-trialStart < .75 end
    Screen('DrawTexture',w,probeScreen);
    vbl=Screen('Flip',w);
    %probeStart=GetSecs;
    %while GetSecs-probeStart < .1 end
    Screen('DrawTexture',w,blankScreen);                                %% display probe display
    vbl2=Screen('Flip',w,vbl+.1-1/(REFRESHRATE*2));
    %     WaitSecs(0.1);                                                %% probe display appears for 100ms
    %blankStart=GetSecs;
    %while GetSecs-blankStart < .9 end
                                                                        %% clear the screen
    %     WaitSecs(0.9);                                                %% wait 900ms before test display
    Screen('DrawTexture',w,testScreen);                             %% display test display
    Screen('Flip',w,vbl2+0.9-1/(REFRESHRATE*2));
    startTime = GetSecs;                                                %% start response window
    
    while 1
        [keyIsDown, secs, keyCode] = KbCheck;                           %% check for keypress
        if keyCode(z_key)                                               %% same response
            endTime = GetSecs;                                          %% record time of response
            responseTime = 1000 * (endTime - startTime);                %% get RT of response
            trials(i).response = 0;                                     %% store value of response: '0' = same
            trials(i).RT = responseTime;                                %% store RT
            break;                                                      %% end loop
        elseif keyCode(slash_key)                                       %% different response
            endTime = GetSecs;                                          %% record time of response
            responseTime = 1000 * (endTime - startTime);                %% get RT of response
            trials(i).response = 1;                                     %% store value of response: '1' = different
            trials(i).RT = responseTime;                                %% store RT
            break;                                                      %% end loop
        elseif keyCode(escapeKey)                                       %% if 'esc' key
            ShowCursor;                                                 %% show cursor            
            Screen('CloseAll');                                         %% close all windows
            break;                                                      %% exit to Matlab (will get error message)
        else continue;                                                  %% keep going until you get a valid response
        end %% end  if/elseif/else clause
    end %% end while loop
    
    %% Store SDT Data
    if trials(i).change && trials(i).response                           %% Hit
        trials(i).correct = 1;                                          %% correct trial
        trials(i).hit = 1;                                              %% store value as a hit
        trials(i).miss = 0;                                             %% store null value for miss
        trials(i).falseAlarm = 0;                                       %% store null value for false alarm
        trials(i).correctRejection = 0;                                 %% store null value for correct rejection
        hits = hits + 1;                                                %% increment 'hits' counter
    elseif trials(i).change && ~trials(i).response                      %% Miss
        trials(i).correct = 0;                                          %% incorrect trial
        trials(i).hit = 0;                                              %% store null value for hit
        trials(i).miss = 1;                                             %% store value as a miss
        trials(i).falseAlarm = 0;                                       %% store null value for false alarm
        trials(i).correctRejection = 0;                                 %% store null value for correct rejection
        misses = misses + 1;                                            %% increment 'misses' counter
    elseif ~trials(i).change && trials(i).response                      %% False Alarm
        trials(i).correct = 0;                                          %% incorrect trial
        trials(i).hit = 0;                                              %% store null value for hit
        trials(i).miss = 0;                                             %% store null value for miss
        trials(i).falseAlarm = 1;                                       %% store value as a false alarm
        trials(i).correctRejection = 0;                                 %% store null value for correct rejection
        falseAlarms = falseAlarms + 1;                                  %% increment 'false alarms' counter
    elseif ~trials(i).change && ~trials(i).response                     %% Correct Rejection
        trials(i).correct = 1;                                          %% correct trial
        trials(i).hit = 0;                                              %% store null value for hit
        trials(i).miss = 0;                                             %% store null value for miss
        trials(i).falseAlarm = 0;                                       %% store null value for false alarm
        trials(i).correctRejection = 1;                                 %% store value as a correct rejection
        correctRejections = correctRejections + 1;                      %% increment 'correct rejections' counter
    end %% end if/elseif(s) clause
    
    if currTrial==practiceTrials                                            %% end of practice trials
        message2='End of practice trials.  Press the spacebar to continue'; %% end of practice trials message
        Screen('DrawText',w,message2,XCentre./2,YCentre,black);
        Screen('Flip',w);
        currBlock = 1;                                                      %% start experimental trials
        currBlockTrial = 1;                                                 %% reset block counter
        hits = 0;                                                           %% reset SDT values
        misses = 0;
        falseAlarms = 0;
        correctRejections = 0;
    else
        currBlockTrial = currBlockTrial + 1;                                %% increment block counter
    end %% end if clause
    
    currTrial = currTrial + 1;                                          %% increment trial counter
    
end  %% end for loop

endMessage='All done!!!  Please inform the experimenter.';
Screen('DrawText',w,endMessage,XCentre./2,YCentre,black);
Screen('Flip',w);
pause(1)
KbWait;
Screen('CloseAll');

%%************************************************************************
%% Record Results
%%************************************************************************

data_file = fopen(['VSTMCapacity_' num2str(subNum) '.out'],'a');        %% open data file for writing
fprintf(data_file,'Subject\tBlock\tBlockTrial\tTrial\tSetSize\t');      %% add headers
fprintf(data_file,'Change?\tResponse\tRT\tCorrect\t');
fprintf(data_file,'Hit\tMiss\tFalseAlarm\tCorrectRejection\n');

for i = 1:numTrials                                                     %% print out trial data
    fprintf(data_file,'%d\t%d\t%d\t%d\t%d\t%d\t%d\t%f\t%d\t%d\t%d\t%d\t%d\n', ...
            num2str(subNum),trials(i).block,trials(i).blockTrial,trials(i).trial, ...
            trials(i).setsize,trials(i).change,trials(i).response,       ...
            trials(i).RT,trials(i).correct,trials(i).hit,trials(i).miss, ...
            trials(i).falseAlarm,trials(i).correctRejection);
end %% end for loop



%%************************************************************************
%% Compute K
%%************************************************************************

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Pashler's Formula
%%% K = [S x (H - F)] / (1 - F)
%%% K = capacity, S = set size
%%% H = hit rate: [hits / (hits + misses)] 
%%% F = false alarm rate: [false alarms / (false alarms + correct rejections)] 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

hitRate         = hits / (hits + misses)                                %% compute hitrate
falseAlarmRate  = falseAlarms / (falseAlarms + correctRejections)       %% computer false alarm rate
capacity = (setsize*(hitRate-falseAlarmRate)) / (1.0 - falseAlarmRate)  %% compute K

fprintf(data_file,'\nHits:\t%d\tMisses:\t%d\tHitRate:\t%f\n', ...       %% print out hits, misses, hitrate to data file
                   hits, misses, hitRate);
fprintf(data_file,'FAs:\t%d\tCRs:\t%d\tFARate:\t%f\n',        ...       %% print out false alarms, correct rejections, false alarm rate to data file
                   falseAlarms, correctRejections, falseAlarmRate);
fprintf(data_file,'Capacity:\t%f\n',capacity);                          %% print out capacity to data file

save (['VSTMcapacity_' num2str(subNum) '.mat']);
%%************************************************************************
%% Clean Up
%%************************************************************************
fclose(data_file);                                                      %% close data file

clear all;
clc;
ShowCursor;
